/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew;

import java.io.File;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.lms.vew.daten.RDSMeldungsDaten;
import de.bsvrz.kex.lms.vew.daten.RDSStatus.RDS_StatusEN;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * SWE-Interne Repr&auml;sentation einer RDS-Meldung.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class RDSMeldung implements ClientSenderInterface, ClientReceiverInterface {
	/**
	 * XML-Tag RDS-Meldung.
	 */
	public static final String XML_ELEMENT_MELDUNG = "IFN";

	/**
	 * DebugLogger für Debug-Ausgaben.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Die Datei, in der die Meldung enthalten ist.
	 */
	private File meldungsDatei;

	/**
	 * der interne Status der Meldung.
	 */
	private MeldungsStatus status = MeldungsStatus.VERSENDET;

	/**
	 * Das zugeh&ouml;rige Systemobjekt.
	 */
	private final SystemObject systemObject;

	/**
	 * Flag, ob die Datenanmeldung beim DaV erfolgt ist.
	 */
	private boolean istAngemeldet = false;

	/**
	 * Synchronisationsobjekt f&uuml;r die Datenanmeldung.
	 */
	private final Object anmeldungsLock = new Object();

	/** Wartezeit auf erfolgreiche Anmeldebest&auml;tigung. */
	private final long warteZeitAnmeldung = TimeUnit.SECONDS.toMillis(20);

	/** Zeitpunkt des letzten Versandes. */
	private long letzterVersand = 0;

	/** die Daten der Meldung. */
	private RDSMeldungsDaten meldungsDaten;

	/** Datenspezifikation f&uuml;r die empfangenen Daten. */
	private DavSendeAnmeldung dataEmpfangen = null;

	/** Datenspezifikation f&uuml;r die empfangenen Daten als Parameter. */
	private DavSendeAnmeldung dataParameter = null;

	/** Datenspezifikation f&uuml;r die zu versendenden Daten. */
	private DavSendeAnmeldung dataVersand;

	/** Datenspezifikation f&uuml;r die 'versendet' Daten. */
	private DavSendeAnmeldung dataVersendet;

	/** Flag, ob f&uuml;r die Meldung Daten vom DaV empfangen wurden. */
	private boolean davDatenOK = false;

	/** Flag, ob die Meldung versendet wurde. */
	private boolean versendet = false;

	/** Kopie der DAV-Daten zum Versenden. */
	private Data davDatenVersenden = null;

	/**
	 * Erzeugt eine neue RDS-Meldung aus einem Systemobjekt.
	 *
	 * @param object
	 *            Systemobjekt
	 */
	public RDSMeldung(final SystemObject object) {
		this(object, new RDSMeldungsDaten());
	}

	/**
	 * Erzeugt eine neue RDS-Meldung aus einem Systemobjekt und Meldungsdaten.
	 *
	 * @param object
	 *            Systemobjekt
	 * @param daten
	 *            Meldungsdaten
	 */
	public RDSMeldung(final SystemObject object, final RDSMeldungsDaten daten) {
		systemObject = object;
		meldungsDaten = daten;
	}

	/**
	 * Aktualisiert die Daten der Meldung.
	 *
	 * @param data
	 *            DaV-Daten
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	public void aktualisieren(final Data data) throws RDSException {
		if (data == null) {
			throw new RDSException("Aktualisierung '" + systemObject + "': Keine Daten");
			// debug.warning(systemObject.getPid() + ": Keine Daten");
			// return;
		}

		meldungsDaten.aktualisieren(data);
	}

	/**
	 * Datenverteileranmeldung f&uuml;r empfangene Meldungen.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	public void aktualisierenDav(final ClientDavInterface dav) throws RDSException {
		final DataModel model = dav.getDataModel();

		// aktuelle Daten lesen
		aktualisieren(dav.getData(systemObject,
				new DataDescription(model.getAttributeGroup("atg.rdsMeldung"), model.getAspect("asp.parameterSoll")), 0)
				.getData());
	}

	/**
	 * Datenverteileranmeldung f&uuml;r empfangene Meldungen.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	public void datenAktualisierenDav(final ClientDavInterface dav) throws RDSException {
		final DataModel model = dav.getDataModel();

		// aktuelle Daten lesen
		Data data = dav.getData(systemObject,
				new DataDescription(model.getAttributeGroup("atg.rdsMeldung"), model.getAspect("asp.parameterSoll")), 0)
				.getData();

		if (data == null) {
			data = dav.getData(systemObject,
					new DataDescription(model.getAttributeGroup("atg.rdsMeldung"), model.getAspect("asp.rdsSenden")), 0)
					.getData();
		}

		if (data == null) {
			throw new RDSException("keine aktuellen Daten unter den Aspekten 'asp.parameterSoll' und 'asp.rdsSenden'");
		}

		aktualisieren(data);
	}

	/**
	 * Datenverteileranmeldung f&uuml;r empfangene Meldungen.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	public void anmeldenDavEmpfang(final ClientDavInterface dav) throws RDSException {

		if (istAngemeldet) {
			return;
		}

		final DataModel model = dav.getDataModel();

		dataEmpfangen = new DavSendeAnmeldung(model.getAttributeGroup("atg.rdsMeldung"),
				model.getAspect("asp.rdsEmpfangen"));

		dataParameter = new DavSendeAnmeldung(model.getAttributeGroup("atg.rdsMeldung"),
				model.getAspect("asp.parameterVorgabe"));

		// if(!neu) {
		// // aktuelle Daten lesen
		// aktualisieren(dav.getData(_systemObject,
		// new DataDescription(model.getAttributeGroup("atg.rdsMeldung"),
		// model.getAspect( "asp.parameterSoll")),
		// 0).getData());
		// }

		try {
			RDSMeldung.LOGGER.fine("beginne Datenanmeldung Empfangsdaten: " + this);

			synchronized (anmeldungsLock) {
				dav.subscribeSender(this, systemObject, dataEmpfangen, SenderRole.source());
				dav.subscribeSender(this, systemObject, dataParameter, SenderRole.sender());

				// warte auf Anmeldebestätigung
				RDSMeldung.LOGGER.fine("beginne Wartezeit auf Bestätigung der Datenanmeldung: " + this);

				try {
					anmeldungsLock.wait(warteZeitAnmeldung);
				} catch (final InterruptedException e) {
					// wird nicht behandelt
				}
			}

			RDSMeldung.LOGGER.fine("Ende Warten auf Bestätigung der Datenanmeldung: " + this);

			if (!istAngemeldet) {
				throw new RDSException("Die Meldung " + systemObject.getPid()
						+ " konnte nicht erfolgreich am DaV angemeldet werden" + " (es erfolgte innerhalb von "
				+ (warteZeitAnmeldung / 1000) + "s keine Rückmeldung über die Sendeanmeldung)");
			}

		} catch (final OneSubscriptionPerSendData e) {
			throw new RDSException("Fehler bei DAV Anmeldung: Objekt: " + systemObject + " (" + e.getMessage() + ")");
		}
	}

	/**
	 * Datenverteilerabmeldung f&uuml;r empfangene Meldungen.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	public void abmeldenDavEmpfang(final ClientDavInterface dav) throws RDSException {
		dav.unsubscribeSender(this, systemObject, dataEmpfangen);
		dav.unsubscribeSender(this, systemObject, dataParameter);
	}

	/**
	 * Datenverteileranmeldung f&uuml;r zu versendende Meldungen.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	public void anmeldenDavVersand(final ClientDavInterface dav) throws RDSException {

		if (istAngemeldet) {
			return;
		}

		final DataModel model = dav.getDataModel();

		dataVersand = new DavSendeAnmeldung(model.getAttributeGroup("atg.rdsMeldung"),
				model.getAspect("asp.rdsSenden"));

		dataVersendet = new DavSendeAnmeldung(model.getAttributeGroup("atg.rdsMeldung"),
				model.getAspect("asp.rdsVersendet"));

		// _debug.finer("Anmeldung zum Senden von Daten an den DaV als QUELLE :"
		// +
		// "(Objekt: " + meldung +
		// ") (ATG: " + dataDescription.getAttributeGroup().getName() +
		// ") (ASP: " + dataDescription.getAspect().getName() +
		// ") (OT: " + meldung.get_systemObject().getType().getName() + ")");

		final SystemObject[] objekte = new SystemObject[] { systemObject };

		try {
			dav.subscribeSender(this, systemObject, dataVersendet, SenderRole.source());
			// dav.subscribeSender(this, _systemObject, _dataParameter,
			// SenderRole.sender());
		} catch (final OneSubscriptionPerSendData e) {
			throw new RDSException(e.getMessage());
		}

		// aktuelle Daten aus Parameter-Datensatz lesen
		try {
			aktualisieren(dav.getData(systemObject, new DataDescription(model.getAttributeGroup("atg.rdsMeldung"),
					model.getAspect("asp.parameterSoll")), 0).getData());
		} catch (final RDSException e) {
			// das ist hier kein Fehler:
			// Die Meldungen zum Versand werden zunächst von der SWE RdsTmc
			// mit Daten unter den Aspekt 'rds.generiert' angelegt, der hier
			// nicht benutzt wird.
		}

		dav.subscribeReceiver(this, objekte, dataVersand, new ReceiveOptions(false), // keine
				// Deltas,
				// keine
				// nachgelieferten
				// Daten
				ReceiverRole.receiver());
	}

	/**
	 * Datenverteilerabmeldung f&uuml;r zu versendende Meldungen.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	public void abmeldenDavVersand(final ClientDavInterface dav) throws RDSException {
		final SystemObject[] objekte = new SystemObject[] { systemObject };

		dav.unsubscribeSender(this, systemObject, dataVersendet);
		dav.unsubscribeReceiver(this, objekte, dataVersand);
	}

	/**
	 * Erzeugt eine PID aus den Meldungsdaten.
	 *
	 * @return PID
	 */
	public String bildePid() {
		return meldungsDaten.bildePid();
	}

	@Override
	public void dataRequest(final SystemObject object, final DataDescription dataDescription, final byte state) {
		boolean anmeldungOK = false;

		RDSMeldung.LOGGER.finer("dataRequest: " + this + ", DataDescription: " + dataDescription);
		if (dataDescription == dataParameter) {
			dataParameter.setzeSendenOK(state == ClientSenderInterface.START_SENDING);
		}

		if (dataDescription == dataEmpfangen) {
			// die Daten werden auch gesendet, wenn kein Empfänger angemeldet
			// ist
			dataEmpfangen.setzeSendenOK(
					(state == ClientSenderInterface.START_SENDING) || (state == ClientSenderInterface.STOP_SENDING));
		}

		// wir bescheiden uns mit der Anmeldung an der Parametrierung
		if (dataParameter != null) {
			anmeldungOK = dataParameter.sendenOK();
		}

		if (anmeldungOK && !istAngemeldet) {
			istAngemeldet = true;
			RDSMeldung.LOGGER
					.finer("dataRequest: " + this + " Lock-Freigabe: " + istAngemeldet + ", " + anmeldungsLock);
			if (anmeldungsLock != null) {
				synchronized (anmeldungsLock) {
					anmeldungsLock.notify();
				}
			}
		}
	}

	/**
	 * Vervollst&auml;ndigt die Daten der Meldung.
	 *
	 * InfoGUID (IID): Identifikation, aktuelle Systemzeit (Millisekunden)
	 * IDOrganisation (ORG): aus RDS-Meldung oder wenn nicht gesetzt aus
	 * Parameter atg.rdsKommunikationsParameter.EigeneID. IDNumber (NUM):
	 * inkrementierte laufende Nummer aus
	 * atg.rdsKommunikationsParameter.LaufendeNummer ReminderTime (REM) wird
	 * gebildet aus aktueller Zeit + Wert des Attributes
	 * atg.rdsKommunikationsParameter.SendeWiederholung
	 *
	 * @param parameter
	 *            LMS-Parameter
	 */
	public void datenvervollstaendigen(final LMSParameter parameter) {
		if ((meldungsDaten.getIdentifikation() == null) || (meldungsDaten.getIdentifikation().length() == 0)) {
			meldungsDaten.setzeIdentifikation(String.valueOf(System.currentTimeMillis()));
		}

		if ((meldungsDaten.getIDOrganisation() == null) || (meldungsDaten.getIDOrganisation().length() == 0)) {
			meldungsDaten.setIDOrganisation(parameter.getEigeneID());
		}

		meldungsDaten.setIDNumber(String.valueOf(parameter.getLaufendeNummer()));
	}

	/**
	 * Erzeugt das XML-Element f&uuml;r die Meldung.
	 *
	 * @param doc
	 *            Dokument
	 * @param root
	 *            Wurzelelement
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	public void erzeugeDocumentElement(final Document doc, final Element root) throws RDSException {
		meldungsDaten.erzeugeDocumentElement(doc, root);
	}

	/**
	 * Gibt den Zeitpunkt des letzten Versandes der Meldung zur&uuml;ck.
	 *
	 * @return Zeitpunkt des letzten Versandes
	 */
	public long getLetzterVersand() {
		return letzterVersand;
	}

	/**
	 * Gibt alle in der Meldung referenzierten Locationcodes zur&uuml;ck.
	 *
	 * @return Liste von Locationcodes
	 */
	public List<Integer> getLocationCodes() {
		return meldungsDaten.getLocationCodes();
	}

	/**
	 * Gibt die zur Meldung geh&ouml;rende Datei zur&uuml;ck.
	 *
	 * @return Datei, in der die Meldung enthalten ist
	 */
	public File getMeldungsDatei() {
		return meldungsDatei;
	}

	/**
	 * Gibt die Daten der Meldung zur&uuml;ck.
	 *
	 * @return Daten der Meldung
	 */
	protected RDSMeldungsDaten getMeldungsDaten() {
		return meldungsDaten;
	}

	/**
	 * Gibt die PID des zugeh&ouml;rigen Systemobjektes zur&uuml;ck.
	 *
	 * @return PID
	 */
	public String getPID() {
		return systemObject.getPid();
	}

	/**
	 * Gibt den RDS-Meldungsstatus zur&uuml;ck.
	 *
	 * @return RDS-Status
	 */
	public RDS_StatusEN getRDSStatus() {
		return meldungsDaten.getMeldungsStatus();
	}

	/**
	 * Gibt den internen Status der Meldung zur&uuml;ck.
	 *
	 * @return Status
	 */
	public MeldungsStatus getStatus() {
		return status;
	}

	/**
	 * Gibt das zugeh&ouml;rige Systemobjekt zur&uuml;ck.
	 *
	 * @return zugeh&ouml;riges Systemobjekt
	 */
	public SystemObject getSystemObject() {
		return systemObject;
	}

	/**
	 * Gibt die Versionsnummer der Meldung zur&uuml;ck.
	 *
	 * @return Versionsnummer
	 */
	public int getVersion() {
		return meldungsDaten.getVersion();
	}

	/**
	 * Test, ob f&uuml;r die Meldung Daten vom DaV empfangen wurden.
	 *
	 * @return true, wenn Daten vom DaV empfangen wurden, sonst false
	 */
	public boolean isDavDatenOK() {
		return davDatenOK;
	}

	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription dataDescription) {
		return true;
	}

	/**
	 * &Uuml;berpr&uuml;ft, ob die Meldung von der SWE implizit aufgehoben wird.
	 * <br>
	 * Die SWE generiert eine implizite Aufhebung wenn der Status nicht
	 * <code>Aufhebung</code> oder <code>Loeschung</code> ist und die
	 * G&uuml;ltigkeit abgelaufen ist.
	 *
	 * @return true, wenn die Meldung aufgehoben wurde, sonst false
	 */
	public boolean istImplizitAufgehoben() {
		if ((meldungsDaten.getMeldungsStatus() != RDS_StatusEN.Aufhebung)
				&& (meldungsDaten.getMeldungsStatus() != RDS_StatusEN.Loeschung) && versendet && !istZeitGueltig()) {
			LogTools.log(RDSMeldung.LOGGER, LmsMsg.InfoMeldungAufgehoben, getPID());
			setRDSStatus(RDS_StatusEN.Aufhebung);
			return true;
		}

		return false;
	}

	/**
	 * Test, ob die Meldung zeitlich g&uuml;ltig ist.
	 *
	 * @return true wenn g&uuml;ltig, sonst false
	 */
	public boolean istZeitGueltig() {
		return meldungsDaten.istZeitGueltig();
	}

	/**
	 * Test, ob die Meldung verarbeitet werden muss. Meldungen mit Status
	 * 'Aufhebung' oder 'L&ouml;schung' werden immer verarbeitet. F&uuml;r alle
	 * anderen Stati wird gepr&uuml;ft, ob die Ablaufzeit bereits erreicht ist.
	 *
	 * @return true, wenn die Version (noch) g&uuml;ltig ist, sonst false
	 */
	public boolean istZuVerarbeiten() {
		return meldungsDaten.istZuVerarbeiten();
	}

	/**
	 * Test, ob die Meldung versendet wurde.
	 *
	 * @return true, wenn versendet, sonst false
	 */
	public boolean isVersendet() {
		return versendet;
	}

	/**
	 * Publiziert die Daten zum DaV.
	 *
	 * @param dav
	 *            Datenverteilerverbindung
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	public void publiziereDAV(final ClientDavInterface dav) throws RDSException {
		try {
			final Data data = dav.createData(dataEmpfangen.getAttributeGroup());
			meldungsDaten.publizieren(data);
			final ResultData meldungsdaten = new ResultData(systemObject, dataEmpfangen, System.currentTimeMillis(),
					data);
			final ResultData meldungsdatenprm = new ResultData(systemObject, dataParameter, System.currentTimeMillis(),
					data);
			if (dataEmpfangen.sendenOK()) {
				RDSMeldung.LOGGER.finest("Sende Daten (empfangen) zum DAV: " + meldungsdaten);
				dav.sendData(meldungsdaten);
			}

			dav.sendData(meldungsdatenprm);

		} catch (final DataNotSubscribedException e) {
			RDSMeldung.LOGGER.error(e.getMessage());
		} catch (final SendSubscriptionNotConfirmed e) {
			RDSMeldung.LOGGER.error(e.getMessage());
		}
	}

	/**
	 * Publiziert die Daten zum DaV, dass die Meldung versendet wurde.
	 *
	 * @param dav
	 *            Datenverteilerverbindung
	 */
	public void publiziereVersendet(final ClientDavInterface dav) {
		try {
			Data data;

			// Benutze Kopie der Daten zum Versand, wenn vorhanden
			if (davDatenVersenden != null) {
				data = davDatenVersenden;
			} else {
				data = dav.createData(dataVersendet.getAttributeGroup());
			}

			meldungsDaten.publizieren(data);

			// Urlasserinformationen setzen
			final Data urlasserData = data.getItem("Urlasser");
			urlasserData.getTextValue("Ursache").setText("an die LMS versendet");
			urlasserData.getTextValue("Veranlasser").setText(dav.getLocalApplicationObject().getName());

			final ResultData meldungsdaten = new ResultData(systemObject, dataVersendet, System.currentTimeMillis(),
					data);
			// ResultData meldungsdatenprm = new
			// ResultData(_systemObject,
			// _dataParameter, System.currentTimeMillis(), data);

			// if(_dataEmpfangen.sendenOK())
			// dav.sendData(meldungsdaten);
			//
			// dav.sendData(meldungsdatenprm);
			RDSMeldung.LOGGER.finest("Sende Daten (versendet) zum DAV: " + meldungsdaten);
			dav.sendData(meldungsdaten);
		} catch (final DataNotSubscribedException e) {
			RDSMeldung.LOGGER.error(e.getMessage());
		} catch (final SendSubscriptionNotConfirmed e) {
			RDSMeldung.LOGGER.error(e.getMessage());
		} catch (final RDSException e) {
			RDSMeldung.LOGGER.error(e.getMessage());
		}
	}

	/**
	 * Setzt den Zeitpunlt des letzten Versandes der Meldung.
	 *
	 * @param versand
	 *            Zeitpunkt letzter Versand
	 */
	public void setLetzterVersand(final long versand) {
		letzterVersand = versand;
	}

	/**
	 * Setzt die zur Meldung geh&ouml;rende Datei.
	 *
	 * @param datei
	 *            Datei, in der die Meldung enthalten ist
	 */
	public void setMeldungsDatei(final File datei) {
		meldungsDatei = datei;
	}

	/**
	 * Setzt die Daten der Meldung.
	 *
	 * @param meldungsdaten
	 *            Daten der Meldung
	 */
	public void setMeldungsDaten(final RDSMeldungsDaten meldungsdaten) {
		meldungsDaten = meldungsdaten;
	}

	/**
	 * Setzt den Status der Meldung.
	 *
	 * @param neuerstatus
	 *            neuer Status
	 */
	public void setRDSStatus(final RDS_StatusEN neuerstatus) {
		meldungsDaten.setMeldungsStatus(neuerstatus);
	}

	/**
	 * Setzt den internen Status der Meldung.
	 *
	 * @param status
	 *            Status
	 */
	public void setStatus(final MeldungsStatus status) {
		this.status = status;
	}

	/**
	 * Setzt das Flag, ob die Meldung versendet wurde.
	 *
	 * @param versendet
	 *            Meldung versendet
	 */
	public void setVersendet(final boolean versendet) {
		this.versendet = versendet;
	}

	@Override
	public String toString() {
		return getPID();
	}

	@Override
	public void update(final ResultData[] results) {
		for (final ResultData datensatz : results) {
			if (datensatz.hasData()) {
				RDSMeldung.LOGGER.finer("Datensatz erhalten für ", datensatz.getObject());
				if (datensatz.getDataDescription().equals(dataVersand)) {
					RDSMeldung.LOGGER.finest("Datensatz: ", datensatz);
					try {
						// Kopie der empfangen Daten zum Versand anlegen, da die
						// interne
						// Repräsentation nicht alle Daten enthält (ATG wurde
						// von Dambach erweitert)
						davDatenVersenden = datensatz.getData().createModifiableCopy();

						aktualisieren(datensatz.getData());
						davDatenOK = true;
						setStatus(MeldungsStatus.AKTUALISIERT);
					} catch (final RDSException e) {
						RDSMeldung.LOGGER.error(e.getMessage());
					}
				}
			}
		}
	}

}
