/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew.daten;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lms.vew.RDSException;

/**
 * RDS-Location Informationen zu einer Location.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class RDSLocation extends AbstractRDSInfoElement {

	/**
	 * Die Koordinaten der Location. Koordinaten im WGS 84 Format in Grad mit 10
	 * mikroGrad Aufl&ouml;sung (1/100000).
	 */
	class RDSLocationKoordinaten extends AbstractRDSInfoElement {
		/**
		 * Coordinate in WGS 84 format in decimal degrees with 10 micro degrees
		 * resolution (1/100000).
		 */
		class RDSLocationKoordinate extends AbstractRDSInfoElement {

			/**
			 * Die Skalierung des TIC-Info -Wertes.
			 */
			static final double TICSKALIERUNG = 100000;

			/**
			 * Der Wert der Koordinate.
			 */
			double koordinatenwert = 0;

			/**
			 * Erzeugt ein neues Objekt.
			 *
			 * @param xmlName
			 *            Name des XML-ELementes
			 * @param davName
			 *            Name des DAV-Attributes
			 */
			public RDSLocationKoordinate(final String xmlName, final String davName) {
				super(xmlName, davName);
			}

			@Override
			public void getDavDaten(final Data davdata) throws RDSException {
				koordinatenwert = davdata.getScaledValue(getDavAttributName()).doubleValue()
						* RDSLocationKoordinate.TICSKALIERUNG;
			}

			@Override
			public void getXmlDaten(final Node node) throws RDSException {
				koordinatenwert = Double.parseDouble(node.getTextContent());
			}

			@Override
			public void setDavDaten(final Data davdata) throws RDSException {
				davdata.getScaledValue(getDavAttributName()).set(koordinatenwert / RDSLocationKoordinate.TICSKALIERUNG);
			}

			@Override
			public void setXmlDaten(final Element element) {
				element.setTextContent(String.valueOf((long) koordinatenwert));
			}

		}

		/**
		 * TIC Info XML Element: LocationLocCoordinates.
		 */
		private static final String XML_ELEMENT_LOCATIONLOCCOORDINATES = "LCO";

		/**
		 * L&auml;nge. TIC Info XML Element: LocationLocCoordinatesX.
		 */
		private static final String XML_ELEMENT_LOCATIONLOCCOORDINATESX = "X";

		/**
		 * Breite. TIC Info XML Element: LocationLocCoordinatesY.
		 */
		private static final String XML_ELEMENT_LOCATIONLOCCOORDINATESY = "Y";

		/**
		 * Erzeugt ein neues Objekt.
		 */
		public RDSLocationKoordinaten() {
			super(RDSLocationKoordinaten.XML_ELEMENT_LOCATIONLOCCOORDINATES, "Koordinaten");
			appendChild(new RDSLocationKoordinate(RDSLocationKoordinaten.XML_ELEMENT_LOCATIONLOCCOORDINATESX,
					"RDSLocationKoordinateX"));
			appendChild(new RDSLocationKoordinate(RDSLocationKoordinaten.XML_ELEMENT_LOCATIONLOCCOORDINATESY,
					"RDSLocationKoordinateY"));
		}

	}

	/**
	 * TIC Info XML Element: LocationLoc, enth&auml;lt alle Informationen zu
	 * einer Location.
	 */
	public static final String XML_ELEMENT_LOCATIONLOC = "LOC";

	/**
	 * Location-Code. TIC Info XML Element: LocationLocCode, Location-Code.
	 * 4294967295 ist ein spezieller Code f&uuml;r Locations mit LocationFormat
	 * = TIC Location.
	 */
	private static final String XML_ELEMENT_LOCATIONLOCCODE = "LCD";

	/**
	 * Strassen-Nummer TIC Info XML Element: LocationLocRoadNumber,
	 * Strassen-Nummer.
	 */
	private static final String XML_ELEMENT_LOCATIONLOCROADNUMBER = "LRN";

	/**
	 * Ausfahrt-Nummer. TIC Info XML Element: LocationLocExitNumber,
	 * Ausfahrt-Nummer.
	 */
	private static final String XML_ELEMENT_LOCATIONLOCEXITNUMBER = "LEN";

	/**
	 * Kilometrierungsangabe. TIC Info XML Element: LocationLocMilestone, Werte
	 * im Format "123" oder "123.45" werden erwartet.
	 */
	private static final String XML_ELEMENT_LOCATIONLOCMILESTONE = "LMS";

	/**
	 * Linien-Location. TIC Info XML Element: LocationLocLinearReference,
	 * Location Code der zugeh&ouml;rigen Linien-Location.
	 */
	private static final String XML_ELEMENT_LOCATIONLOCLINEARREFERENCE = "LLR";

	/**
	 * Gebiets-Location. TIC Info XML Element: LocationLocAreaReference,
	 * Location Code der zugeh&ouml;rigen Gebiets-Location.
	 */
	private static final String XML_ELEMENT_LOCATIONLOCAREAREFERENCE = "LAF";

	/**
	 * N&auml;chste Location in negativer Richtung. TIC Info XML Element:
	 * LocationLocNegativeOffset, Location-Code der n&auml;chsten Location in
	 * negativer Richtung.
	 */
	private static final String XML_ELEMENT_LOCATIONLOCNEGATIVEOFFSET = "LNF";

	/**
	 * N&auml;chste Location in positiver Richtung. TIC Info XML Element:
	 * LocationLocPositiveOffset, Location-Code der n&auml;chsten Location in
	 * positiver Richtung.
	 */
	private static final String XML_ELEMENT_LOCATIONLOCPOSITIVEOFFSET = "LPF";

	/**
	 * Nachrichtentexte.
	 */
	private final RDSLocationCode location = new RDSLocationCode(RDSLocation.XML_ELEMENT_LOCATIONLOCCODE,
			"LocationCode");

	/**
	 * Erzeugt ein neues Objekt.
	 */
	public RDSLocation() {
		super(RDSLocation.XML_ELEMENT_LOCATIONLOC, "");
		appendChild(location);
		appendChild(new RDSText(RDSLocation.XML_ELEMENT_LOCATIONLOCROADNUMBER, "StrassenNummer"));
		appendChild(new RDSText(RDSLocation.XML_ELEMENT_LOCATIONLOCEXITNUMBER, "AusfahrtNummer"));
		appendChild(new RDSTmcOrtsTyp());
		appendChild(new RDSLocationKoordinaten());
		appendChild(new RDSText(RDSLocation.XML_ELEMENT_LOCATIONLOCMILESTONE, "LocationKilometrierung"));
		appendChild(new RDSInteger(RDSLocation.XML_ELEMENT_LOCATIONLOCLINEARREFERENCE, "LocationCodeLinienReferenz"));
		appendChild(new RDSInteger(RDSLocation.XML_ELEMENT_LOCATIONLOCAREAREFERENCE, "LocationCodeGebietsReferenz"));
		appendChild(new RDSInteger(RDSLocation.XML_ELEMENT_LOCATIONLOCNEGATIVEOFFSET, "LocationNextNegativ"));
		appendChild(new RDSInteger(RDSLocation.XML_ELEMENT_LOCATIONLOCPOSITIVEOFFSET, "LocationNextPositiv"));
		appendChild(new RDSNachrichten("LocationNachricht"));
	}

	/**
	 * Gibt den Locationcode zur&uuml;ck.
	 *
	 * @return Locationcode
	 */
	public int getLocationCode() {
		return location.wert;
	}

	@Override
	public void publizieren(final Data davdata) throws RDSException {
		setDavDaten(davdata);
		for (final RDSInfoElement child : kinder) {
			child.publizieren(davdata);
		}
	}
}
