/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew.daten;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lms.vew.LmsMsg;
import de.bsvrz.kex.lms.vew.RDSException;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Verkehrsinformationen einer Version einer RDS-Meldung.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class RDSVerkehrsInformation extends AbstractRDSInfoElement {

	/**
	 * Ereignis-Informationen.
	 */
	class RDSEreignis extends AbstractRDSInfoElement {

		/**
		 * Ereignis-Daten.
		 * <p>
		 * Zuordnung der Ereigniskodierung:
		 * </p>
		 * <p>
		 * Vorhersage: Laut Aussage GEWI (Hr. Kuske) ist die Vorhersagekodierung
		 * beim Erzeugen der Meldung nicht auf Konsistenz gepr&uuml;ft, d.h. das
		 * Flag 'AttributeForecast' in den Zusatzinformationen ist
		 * unabh&auml;ngig von der Kodierung in den Ereignisdaten. Lt. DIN EN
		 * ISO 14819-2 haben die Vorhersagen die Ereignis-Kategorie 32 bis 39.
		 * Es besteht also im Moment nur die M&ouml;glichkeit, eine Vorhersage
		 * an dem gesetzten Flag UND der Ereignis-Kategorie 32-39 festzustellen.
		 * </p>
		 */
		class RDSEreignisDaten extends AbstractRDSInfoElement {
			/**
			 * Quantit&auml;tsangaben zum Ereignis.
			 */
			class RDSEreignisQuantitaet extends AbstractRDSInfoElement {
				/**
				 * TIC Info XML Element: EventDataQuantities.
				 */
				public static final String XML_ELEMENT_EVENTDATAQUANTITIES = "EQT";

				/**
				 * Inhalt des zus&auml;tzlichen Wertes. TIC Info XML Element:
				 * EventDataQuantitiesContent
				 */
				private static final String XML_ELEMENT_EVENTDATAQUANTITIESCONTENT = "EQC";

				/**
				 * Einheit des zus&auml;tzlichen Wertes. TIC Info XML Element:
				 * EventDataQuantitiesUnitQualifier
				 */
				private static final String XML_ELEMENT_EVENTDATAQUANTITIESUNITQUALIFIER = "EQU";

				/**
				 * TIC Info XML Element: EventDataQuantitiesQualifier.
				 */
				private static final String XML_ELEMENT_EVENTDATAQUANTITIESQUALIFIER = "EQL";

				/**
				 * Konstruktor.
				 */
				public RDSEreignisQuantitaet() {
					super(RDSEreignisQuantitaet.XML_ELEMENT_EVENTDATAQUANTITIES, "");
					appendChild(new RDSText(RDSEreignisQuantitaet.XML_ELEMENT_EVENTDATAQUANTITIESCONTENT,
							"QuantitätsWert"));
					appendChild(new RDSText(RDSEreignisQuantitaet.XML_ELEMENT_EVENTDATAQUANTITIESUNITQUALIFIER,
							"QuantitätsEinheit"));
					appendChild(new RDSQuantitaetsKennung(
							RDSEreignisQuantitaet.XML_ELEMENT_EVENTDATAQUANTITIESQUALIFIER, "QuantitätsKennung"));
				}
			}

			/**
			 * TIC Info XML Element: EventData, enth&auml;lt die Daten zum
			 * Ereignis.
			 */
			protected static final String XML_ELEMENT_EVENTDATA = "EDA";
			/**
			 * numerischer Wert der Ereignis-Kategorie
			 * <p>
			 * TIC Info XML Element: EventDataCategory, numerischer Wert der
			 * Ereignis-Kategorie
			 * </p>
			 * <p>
			 * 1 bis 39: Ereignis-Kategorie entsprechend DIN EN ISO 14819-2
			 * (Event List Section, Spalte 'C'), Attribut &lt;verweis
			 * pid="att.rdsEreignisCode"/&gt; ist g&uuml;ltig.
			 * </p>
			 * <p>
			 * ZUSATZINFORMATION (s.o): 32 bis 39: sind Vorhersagen entsprechend
			 * DIN EN ISO 14819-2
			 * </p>
			 * <p>
			 * 40 bis 49: nicht benutzt
			 * </p>
			 * <p>
			 * 50: spezielle Kategorie, kennzeichnet dieses Ereignis als eine
			 * Empfehlung entsprechend DIN EN ISO 14819-2 (Tabelle
			 * 'SUPPLEMENTARY INFORMATION LIST', Spalte 'Code'), Attribut
			 * &lt;verweis pid="att.rdsEmpfehlungsCode"/&gt; ist g&uuml;ltig
			 * </p>
			 * TIC Info XML Element: EventDataCategory, numerischer Wert der
			 * Ereignis-Kategorie
			 */
			private static final String XML_ELEMENT_EVENTDATACATEGORY = "ECT";

			/**
			 * TIC Info XML Element: EventDataCode, Codenummer des Ereignisses
			 * entsprechend DIN EN ISO 14819-2. Das Attribut ist g&uuml;ltig bei
			 * 'att.rdsEreignisKategorie=1-39'.
			 */
			private static final String XML_ELEMENT_EVENTDATACODE = "ECO";
			/**
			 * TIC Info XML Element: EventDataDurationType.
			 */
			private static final String XML_ELEMENT_EVENTDATADURATIONTYPE = "EDT";

			/** Ereignis-Kategorie. */
			RDSZustand ereignisKategorie = new RDSZustand(RDSEreignisDaten.XML_ELEMENT_EVENTDATACATEGORY,
					"EreignisKategorie");

			/** Ereignis-Code. */
			RDSZustand ereignisCode = new RDSZustand(RDSEreignisDaten.XML_ELEMENT_EVENTDATACODE, "EreignisCode");

			/**
			 * Empfehlungs-Code TIC Info XML Element: EventDataCode, Codenummer
			 * der Empfehlung entsprechend DIN EN ISO 14819-2. Das Attribut ist
			 * g&uuml;ltig bei 'att.rdsEreignisKategorie=50'.
			 */
			RDSZustand empfehlungsCode = new RDSZustand(RDSEreignisDaten.XML_ELEMENT_EVENTDATACODE, "EmpfehlungsCode");
			/**
			 * Vorhersage-Code TIC Info XML Element: EventDataCode, Codenummer
			 * der Vorhersage entsprechend DIN EN ISO 14819-2. Das Attribut ist
			 * g&uuml;ltig bei 'att.rdsAttribute.Vorhersage=ja'.
			 */
			RDSZustand vorhersageCode = new RDSZustand(RDSEreignisDaten.XML_ELEMENT_EVENTDATACODE, "VorhersageCode");

			/** Dauer des Ereignisses. */
			RDSZustand ereignisDauer = new RDSZustand(RDSEreignisDaten.XML_ELEMENT_EVENTDATADURATIONTYPE,
					"EreignisDauer");

			/**
			 * Quantit&auml;tsangaben zum Ereignis.
			 */
			List<RDSEreignisQuantitaet> ereignisQuantitaeten = new ArrayList<RDSEreignisQuantitaet>();

			/**
			 * Ereigniscode TIC.
			 */
			RDSZustand ereignisCodeTIC = new RDSZustand(RDSEreignisDaten.XML_ELEMENT_EVENTDATACODE, "");

			/**
			 * Konstruktor.
			 */
			public RDSEreignisDaten() {
				super(RDSEreignisDaten.XML_ELEMENT_EVENTDATA, "");
				appendChild(ereignisCodeTIC);
				appendChild(ereignisKategorie);
				appendChild(ereignisDauer);
			}

			@Override
			public void aktualisieren(final Data davdata) throws RDSException {
				ereignisKategorie.aktualisieren(davdata);
				ereignisCode.aktualisieren(davdata);
				empfehlungsCode.aktualisieren(davdata);
				vorhersageCode.aktualisieren(davdata);
				ereignisDauer.aktualisieren(davdata);

				final Data.Array qarray = davdata.getArray("EreignisQuantität");
				final int anzahl = qarray.getLength();
				ereignisQuantitaeten.clear();
				for (int i = 0; i < anzahl; i++) {
					final RDSEreignisQuantitaet nd = new RDSEreignisQuantitaet();
					nd.aktualisieren(qarray.getItem(i));
					ereignisQuantitaeten.add(nd);
				}
			}

			@Override
			public void einlesenXML(final Node node) throws RDSException {
				super.einlesenXML(node);
				final NodeList nl = node.getChildNodes();
				for (int i = 0; i < nl.getLength(); i++) {
					final Node n = nl.item(i);
					if (n.getNodeName().equals(RDSEreignisQuantitaet.XML_ELEMENT_EVENTDATAQUANTITIES)) {
						final RDSEreignisQuantitaet nd = new RDSEreignisQuantitaet();
						nd.einlesenXML(n);
						ereignisQuantitaeten.add(nd);
					}
				}
			}

			@Override
			public Element erzeugeDocumentElement(final Document doc, final Element parent) throws RDSException {
				final Element oelement = doc.createElement(getXmlElementName());
				parent.appendChild(oelement);
				setXmlDaten(oelement);

				ereignisDauer.erzeugeDocumentElement(doc, oelement);
				ereignisKategorie.erzeugeDocumentElement(doc, oelement);

				// Ereigniscode extrahieren
				if ((ereignisKategorie.wert >= 1) && (ereignisKategorie.wert <= 39)) {
					ereignisCodeTIC.wert = ereignisCode.wert;
				} else if (ereignisKategorie.wert == 50) {
					ereignisCodeTIC.wert = empfehlungsCode.wert;
				} else if (vorhersageJaNein.wert) {
					ereignisCodeTIC.wert = vorhersageCode.wert;
				} else {
					throw new RDSException("Der Ereigniscode kann nicht zugeordnet werden");
				}

				ereignisCodeTIC.erzeugeDocumentElement(doc, oelement);

				final Element em = doc.createElement(RDSEreignisQuantitaet.XML_ELEMENT_EVENTDATAQUANTITIES);
				oelement.appendChild(em);
				for (final RDSEreignisQuantitaet q : ereignisQuantitaeten) {
					q.erzeugeDocumentElement(doc, em);
				}

				return oelement;

			}

			@Override
			public void publizieren(final Data davdata) throws RDSException {
				ereignisKategorie.publizieren(davdata);
				ereignisDauer.publizieren(davdata);

				// Quantitaeten publizieren
				final Data.Array array = davdata.getArray("EreignisQuantität");
				final int anzahl = ereignisQuantitaeten.size();
				array.setLength(anzahl);
				for (int i = 0; i < anzahl; i++) {
					ereignisQuantitaeten.get(i).publizieren(array.getItem(i));
				}

				// alle Codes löschen und publizieren
				ereignisCode.wert = empfehlungsCode.wert = vorhersageCode.wert = 0;
				ereignisCode.publizieren(davdata);
				empfehlungsCode.publizieren(davdata);
				vorhersageCode.publizieren(davdata);

				// TIC-Ereigniscode auf richtigen DaV-Ereigniscode zuweisen
				if ((ereignisKategorie.wert >= 1) && (ereignisKategorie.wert <= 39)) {
					// Vorhersagen werden am gesetzten Flag und Kategorie 32-39
					// festgemacht (s.o)
					// Änderung 10.01.2011: Nach Aussage GEWI besteht kein Bezug
					// des Flags
					// 'vorhersageJaNein' zum Inhalt der Meldung. d.h. das Flag
					// steht für sich als Zusatzinformation und darf hier nicht
					// bewertet werden
					if ((ereignisKategorie.wert >= 31)) {
						// && (vorhersageJaNein.wert)) {
						// Vorhersage
						vorhersageCode.wert = ereignisCodeTIC.wert;
						vorhersageCode.publizieren(davdata);
					} else {
						ereignisCode.wert = ereignisCodeTIC.wert;
						ereignisCode.publizieren(davdata);
					}
				} else if (ereignisKategorie.wert == 50) {
					empfehlungsCode.wert = ereignisCodeTIC.wert;
					empfehlungsCode.publizieren(davdata);
				} else {
					throw new RDSException("Der Ereigniscode kann nicht zugeordnet werden");
				}
			}
		}

		/**
		 * TIC Info XML Element: Event, enth&auml;lt alle Daten zu den
		 * Ereignissen.
		 */
		private static final String XML_ELEMENT_EVENT = "EVT";

		/**
		 * Die Information ist bi-direktional (ja) oder nicht (nein) TIC Info
		 * XML Element: EventBiDirectionFlag.
		 */
		private static final String XML_ELEMENT_EVENTBIDIRECTIONFLAG = "EBF";

		/**
		 * Tabelle, nach der das Ereignis zu interpretieren ist. TIC Info XML
		 * Element: EventType.
		 */
		private static final String XML_ELEMENT_EVENTTYPE = "ETP";

		/**
		 * Nummer der Ereignistabelle TIC Info XML Element: EventTableNumber,
		 * Nummer der benutzten Ereignistabelle (0: unbekannt, 1 bis 65535:
		 * definiert durch GEWI).
		 */
		private static final String XML_ELEMENT_EVENTTABLENUMBER = "ETN";

		/**
		 * Version der Ereignistabelle TIC Info XML Element: EventTableVersion,
		 * textuelle Angabe der Version der Ereignistabelle.
		 */
		private static final String XML_ELEMENT_EVENTTABLEVERSION = "ETV";
		/**
		 * TIC Info XML Element: EventTrafficTypes, pro Ereignis muss mindestens
		 * ein Typ gesetzt sein.
		 */
		private static final String XML_ELEMENT_EVENTTRAFFICTYPES = "TTY";

		/**
		 * Art des Verkehrsereignisses.
		 */
		List<RDSZustand> ereignisTypen = new ArrayList<RDSZustand>();

		/**
		 * Liste der Ereignisdaten.
		 */
		List<RDSEreignisDaten> ereignisDaten = new ArrayList<RDSEreignisDaten>();

		/**
		 * Erzeugt eine neue Instanz.
		 */
		public RDSEreignis() {
			super(RDSEreignis.XML_ELEMENT_EVENT, "Ereignis");
			appendChild(new RDSInteger(RDSEreignis.XML_ELEMENT_EVENTTYPE, "EreignisKodierung"));
			appendChild(new RDSInteger(RDSEreignis.XML_ELEMENT_EVENTTABLENUMBER, "EreignisTabelleNummer"));
			appendChild(new RDSText(RDSEreignis.XML_ELEMENT_EVENTTABLEVERSION, "EreignisTabelleVersion"));
			appendChild(new RDSBoolean(RDSEreignis.XML_ELEMENT_EVENTBIDIRECTIONFLAG, "EreignisInBeidenRichtungen"));
		}

		@Override
		public void aktualisieren(final Data davdata) throws RDSException {
			super.aktualisieren(davdata);
			final Data data = davdata.getItem(getDavAttributName());

			Data.Array darray = data.getArray("EreignisTyp");
			int anzahl = darray.getLength();
			ereignisTypen.clear();
			for (int i = 0; i < anzahl; i++) {
				final RDSZustand ereignistyp = new RDSZustand(RDSEreignis.XML_ELEMENT_EVENTTRAFFICTYPES, "");
				ereignistyp.aktualisieren(darray.getItem(i));
				ereignisTypen.add(ereignistyp);
			}

			darray = data.getArray("EreignisDaten");
			anzahl = darray.getLength();
			ereignisDaten.clear();
			for (int i = 0; i < anzahl; i++) {
				final RDSEreignisDaten nd = new RDSEreignisDaten();
				nd.aktualisieren(darray.getItem(i));
				ereignisDaten.add(nd);
			}
		}

		@Override
		public void einlesenXML(final Node node) throws RDSException {
			super.einlesenXML(node);
			final NodeList nl = node.getChildNodes();
			for (int i = 0; i < nl.getLength(); i++) {
				final Node n = nl.item(i);
				if (n.getNodeName().equals(RDSEreignis.XML_ELEMENT_EVENTTRAFFICTYPES)) {
					final RDSZustand ereignistyp = new RDSZustand(RDSEreignis.XML_ELEMENT_EVENTTRAFFICTYPES, "");
					ereignistyp.einlesenXML(n);
					ereignisTypen.add(ereignistyp);
				}

				if (n.getNodeName().equals(RDSEreignisDaten.XML_ELEMENT_EVENTDATA)) {
					final RDSEreignisDaten nd = new RDSEreignisDaten();
					nd.einlesenXML(n);
					ereignisDaten.add(nd);
				}
			}
		}

		@Override
		public Element erzeugeDocumentElement(final Document doc, final Element parent) throws RDSException {
			final Element oelement = super.erzeugeDocumentElement(doc, parent);

			for (final RDSZustand ereignistyp : ereignisTypen) {
				ereignistyp.erzeugeDocumentElement(doc, oelement);
			}

			for (final RDSEreignisDaten ereignisdaten : ereignisDaten) {
				ereignisdaten.erzeugeDocumentElement(doc, oelement);
			}

			return oelement;
		}

		@Override
		public void publizieren(final Data davdata) throws RDSException {
			super.publizieren(davdata);
			final Data data = davdata.getItem(getDavAttributName());

			Data.Array array = data.getArray("EreignisTyp");
			int anzahl = ereignisTypen.size();
			array.setLength(anzahl);
			for (int i = 0; i < anzahl; i++) {
				ereignisTypen.get(i).publizieren(array.getItem(i));
			}

			// Ereignisdaten testweise publizieren, Fehler werden geloggt, nur
			// fehlerfreie Daten werden
			// weiter genutzt
			array = data.getArray("EreignisDaten");
			final List<RDSEreignisDaten> testliste = new ArrayList<RDSVerkehrsInformation.RDSEreignis.RDSEreignisDaten>(
					ereignisDaten);
			final List<RDSEreignisDaten> okliste = new ArrayList<RDSVerkehrsInformation.RDSEreignis.RDSEreignisDaten>();
			anzahl = testliste.size();
			array.setLength(anzahl);
			for (int i = 0; i < anzahl; i++) {
				try {
					testliste.get(i).publizieren(array.getItem(i));
					okliste.add(testliste.get(i));
				} catch (final RDSException rdse) {
					LogTools.log(Debug.getLogger(), LmsMsg.WarningFehlerEreignisDaten, rdse.getMessage());
				}
			}

			// nur die fehlerfreien werden benutzt
			ereignisDaten = okliste;

			array = data.getArray("EreignisDaten");
			anzahl = ereignisDaten.size();
			array.setLength(anzahl);
			for (int i = 0; i < anzahl; i++) {
				ereignisDaten.get(i).publizieren(array.getItem(i));
			}
		}
	}

	/**
	 * Informationen zur benutzten Location-Tabelle.
	 */
	class RDSLocationTabelleInfo extends AbstractRDSInfoElement {
		/**
		 * TIC Info XML Element: LocationTable, enh&auml;lt alle Informationen
		 * zur benutzten Location-Tabelle.
		 */
		private static final String XML_ELEMENT_LOCATIONTABLE = "LCT";

		/**
		 * TIC Info XML Element: LocationTableType.
		 */
		private static final String XML_ELEMENT_LOCATIONTABLETYPE = "LTT";
		/**
		 * TIC Info XML Element: LocationTableVersion.
		 */
		private static final String XML_ELEMENT_LOCATIONTABLEVERSION = "LCV";

		/**
		 * TIC Info XML Element: LocationTableName.
		 */
		private static final String XML_ELEMENT_LOCATIONTABLENAME = "LTA";
		/**
		 * Typ der Location-Tabelle. TIC Info XML Element: LocationTableType.
		 */
		int locationTabelleTyp = 0; // <zustand name="TMC Location Tabelle"
		// wert="0"></zustand>

		/**
		 * Version der Location-Tabelle.
		 */
		String locationTabelleVersion = "nicht gesetzt";

		/**
		 * Name der Locationtabelle.
		 */
		String locationTabelleName = "nicht gesetzt";

		/**
		 * Erzeugt ein neues Objekt aus einem DAV-Datensatz.
		 *
		 * @param rdsdata
		 *            DAV-Daten
		 * @throws RDSException
		 */

		/**
		 * Konstruktor.
		 */
		public RDSLocationTabelleInfo() {
			super(RDSLocationTabelleInfo.XML_ELEMENT_LOCATIONTABLE, "LocationTabelleInfo");
			appendChild(new RDSLocationTabelle());
			appendChild(new RDSText(RDSLocationTabelleInfo.XML_ELEMENT_LOCATIONTABLEVERSION, "LocationTabelleVersion"));
			appendChild(new RDSInteger(RDSLocationTabelleInfo.XML_ELEMENT_LOCATIONTABLETYPE, "LocationTabelleTyp"));
			appendChild(new RDSText(RDSLocationTabelleInfo.XML_ELEMENT_LOCATIONTABLENAME, "LocationTabelleName"));
		}
	}

	/**
	 * Freier Text.
	 */
	class RDSVerkehrText extends AbstractRDSInfoElement {
		/**
		 * TIC Info XML Element: FreeText.
		 */
		private static final String XML_ELEMENT_FREETEXT = "FTX";

		/**
		 * freier Text TIC Info XML Element: FreeTextText.
		 */
		private static final String XML_ELEMENT_FREETEXTTEXT = "FTT";

		/**
		 * Sprachkennung der NachrichtentexteRDS_VerkehrsInformation TIC Info
		 * XML Element: FreeTextLangID, spezifiziert die Sprache der Nachrichten
		 * entsprechend TIC Info Specification: Anhang 'Language Identifier'.
		 */
		private static final String XML_ELEMENT_FREETEXTLANGID = "FTL";

		/**
		 * Konstruktor.
		 */
		public RDSVerkehrText() {
			super(RDSVerkehrText.XML_ELEMENT_FREETEXT, "FreierText");
			appendChild(new RDSText(RDSVerkehrText.XML_ELEMENT_FREETEXTTEXT, "Text"));
			appendChild(new RDSZustand(RDSVerkehrText.XML_ELEMENT_FREETEXTLANGID, "TextSprache"));
		}
	}

	/**
	 * Zus&auml;tzliche Verkehrsmerkmale einer Meldung.
	 */
	class RDSZusatzAttribute extends AbstractRDSInfoElement {
		/**
		 * TIC Info XML Element: Attribute, zus&auml;tzliche Merkmale der
		 * Meldung.
		 */
		private static final String XML_ELEMENT_ATTRIBUTE = "ATT";

		/**
		 * Geographische Relevanz der Meldung TIC Info XML Element:
		 * AttributAreaOfInterest, Geographische Relevanz der Meldung.
		 */
		private static final String XML_ELEMENT_ATTRIBUTAREAOFINTEREST = "AAI";

		/**
		 * Die Information ist eine Vorhersage (ja) oder aktuelle Meldung
		 * (nein). TIC Info XML Element: AttributeForecast, gibt an, ob die
		 * Information eine Vorhersage oder eine aktuelle Meldung ist .
		 */
		private static final String XML_ELEMENT_ATTRIBUTEFORECAST = "AFC";

		/**
		 * Konstruktor.
		 */
		public RDSZusatzAttribute() {
			super(RDSZusatzAttribute.XML_ELEMENT_ATTRIBUTE, "ZusatzAttribute");
			appendChild(new RDSZustand(RDSZusatzAttribute.XML_ELEMENT_ATTRIBUTAREAOFINTEREST, "GeographischeRelevanz"));
			appendChild(vorhersageJaNein);
		}
	}

	/**
	 * TIC Info XML Element: Traffic, enth&auml;lt alle Verkehrsinformationen
	 * zur Version.
	 */
	private static final String XML_ELEMENT_TRAFFIC = "TRA";

	/**
	 * Format der Verkehrsinformationen (f&uuml;r zuk&uuml;nftige
	 * Erweiterungen). TIC Info XML Element: Format, kennzeichnet das Format der
	 * Verkehrsinformationen (reserviert f&uuml;r zuk&uuml;nftige Erweiterungen)
	 */
	private static final String XML_ELEMENT_FORMAT = "TFO";

	/**
	 * TIC Info XML Element: Code, enth&auml;lt die zugeh&ouml;rige ALERT-C
	 * Kodierung der Meldung.
	 */
	private static final String XML_ELEMENT_CODE = "COD";

	/**
	 * Location Informationen.
	 */
	private final RDSLocationDaten locationDaten = new RDSLocationDaten();

	/**
	 * ALERT-C Kodierung.
	 */
	String alertCCode = "";

	/**
	 * Die Information ist eine Vorhersage (ja) oder aktuelle Meldung (nein).
	 * <p>
	 * (Nach Aussage GEWI besteht kein Bezug dieses Flags zum Inhalt der
	 * Meldung. d.h. das Flag steht f&uuml;r sich als Zusatzinformation).
	 */
	private final RDSBoolean vorhersageJaNein = new RDSBoolean(RDSZusatzAttribute.XML_ELEMENT_ATTRIBUTEFORECAST,
			"Vorhersage");

	/**
	 * Erzeugt ein neues Objekt mit Standardwerten.
	 */
	public RDSVerkehrsInformation() {
		super(RDSVerkehrsInformation.XML_ELEMENT_TRAFFIC, "VerkehrsInformationen");
		appendChild(new RDSInteger(RDSVerkehrsInformation.XML_ELEMENT_FORMAT, "RDSFormat"));
		appendChild(new RDSNachrichten("Nachrichten"));
		appendChild(new RDSVerkehrText());
		appendChild(new RDSLocationTabelleInfo());
		appendChild(locationDaten);
		appendChild(new RDSEreignis());
		appendChild(new RDSZusatzAttribute());
		appendChild(new RDSText(RDSVerkehrsInformation.XML_ELEMENT_CODE, "AlertCCode"));
	}

	/**
	 * Gibt alle in der Meldung referenzierten Locationcodes zur&uuml;ck.
	 *
	 * @return Liste von Locationcodes
	 */
	public List<Integer> getLocationCodes() {
		return locationDaten.getLocationCodes();
	}
}
